# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_resource_server_response_content import CreateResourceServerResponseContent
from ..types.get_resource_server_response_content import GetResourceServerResponseContent
from ..types.list_resource_server_offset_paginated_response_content import (
    ListResourceServerOffsetPaginatedResponseContent,
)
from ..types.resource_server import ResourceServer
from ..types.resource_server_authorization_policy import ResourceServerAuthorizationPolicy
from ..types.resource_server_consent_policy_enum import ResourceServerConsentPolicyEnum
from ..types.resource_server_proof_of_possession import ResourceServerProofOfPossession
from ..types.resource_server_scope import ResourceServerScope
from ..types.resource_server_subject_type_authorization import ResourceServerSubjectTypeAuthorization
from ..types.resource_server_token_dialect_schema_enum import ResourceServerTokenDialectSchemaEnum
from ..types.resource_server_token_encryption import ResourceServerTokenEncryption
from ..types.signing_algorithm_enum import SigningAlgorithmEnum
from ..types.update_resource_server_response_content import UpdateResourceServerResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawResourceServersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        identifiers: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]:
        """
        Retrieve details of all APIs associated with your tenant.

        Parameters
        ----------
        identifiers : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            An optional filter on the resource server identifier. Must be URL encoded and may be specified multiple times (max 10).<br /><b>e.g.</b> <i>../resource-servers?identifiers=id1&identifiers=id2</i>

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]
            Resource servers successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "resource-servers",
            method="GET",
            params={
                "identifiers": identifiers,
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListResourceServerOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListResourceServerOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.resource_servers
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    identifiers=identifiers,
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    include_fields=include_fields,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        identifier: str,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateResourceServerResponseContent]:
        """
        Create a new API associated with your tenant. Note that all new APIs must be registered with Auth0. For more information, read <a href="https://www.auth0.com/docs/get-started/apis"> APIs</a>.

        Parameters
        ----------
        identifier : str
            Unique identifier for the API used as the audience parameter on authorization calls. Can not be changed once set.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        enforce_policies : typing.Optional[bool]
            Whether to enforce authorization policies (true) or to ignore them (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateResourceServerResponseContent]
            Resource server successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "resource-servers",
            method="POST",
            json={
                "name": name,
                "identifier": identifier,
                "scopes": convert_and_respect_annotation_metadata(
                    object_=scopes, annotation=typing.Sequence[ResourceServerScope], direction="write"
                ),
                "signing_alg": signing_alg,
                "signing_secret": signing_secret,
                "allow_offline_access": allow_offline_access,
                "allow_online_access": allow_online_access,
                "allow_online_access_with_ephemeral_sessions": allow_online_access_with_ephemeral_sessions,
                "token_lifetime": token_lifetime,
                "token_dialect": token_dialect,
                "skip_consent_for_verifiable_first_party_clients": skip_consent_for_verifiable_first_party_clients,
                "enforce_policies": enforce_policies,
                "token_encryption": convert_and_respect_annotation_metadata(
                    object_=token_encryption,
                    annotation=typing.Optional[ResourceServerTokenEncryption],
                    direction="write",
                ),
                "consent_policy": consent_policy,
                "authorization_details": authorization_details,
                "proof_of_possession": convert_and_respect_annotation_metadata(
                    object_=proof_of_possession,
                    annotation=typing.Optional[ResourceServerProofOfPossession],
                    direction="write",
                ),
                "subject_type_authorization": convert_and_respect_annotation_metadata(
                    object_=subject_type_authorization,
                    annotation=ResourceServerSubjectTypeAuthorization,
                    direction="write",
                ),
                "authorization_policy": convert_and_respect_annotation_metadata(
                    object_=authorization_policy,
                    annotation=typing.Optional[ResourceServerAuthorizationPolicy],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateResourceServerResponseContent,
                    parse_obj_as(
                        type_=CreateResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        id: str,
        *,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetResourceServerResponseContent]:
        """
        Retrieve <a href="https://auth0.com/docs/apis">API</a> details with the given ID.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to retrieve.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetResourceServerResponseContent]
            Resource server successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="GET",
            params={
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetResourceServerResponseContent,
                    parse_obj_as(
                        type_=GetResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete an existing API by ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or the audience of the resource server to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateResourceServerResponseContent]:
        """
        Change an existing API setting by resource server ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to update.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        enforce_policies : typing.Optional[bool]
            Whether authorization policies are enforced (true) or not enforced (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateResourceServerResponseContent]
            Resource server successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "scopes": convert_and_respect_annotation_metadata(
                    object_=scopes, annotation=typing.Sequence[ResourceServerScope], direction="write"
                ),
                "signing_alg": signing_alg,
                "signing_secret": signing_secret,
                "skip_consent_for_verifiable_first_party_clients": skip_consent_for_verifiable_first_party_clients,
                "allow_offline_access": allow_offline_access,
                "allow_online_access": allow_online_access,
                "allow_online_access_with_ephemeral_sessions": allow_online_access_with_ephemeral_sessions,
                "token_lifetime": token_lifetime,
                "token_dialect": token_dialect,
                "enforce_policies": enforce_policies,
                "token_encryption": convert_and_respect_annotation_metadata(
                    object_=token_encryption,
                    annotation=typing.Optional[ResourceServerTokenEncryption],
                    direction="write",
                ),
                "consent_policy": consent_policy,
                "authorization_details": authorization_details,
                "proof_of_possession": convert_and_respect_annotation_metadata(
                    object_=proof_of_possession,
                    annotation=typing.Optional[ResourceServerProofOfPossession],
                    direction="write",
                ),
                "subject_type_authorization": convert_and_respect_annotation_metadata(
                    object_=subject_type_authorization,
                    annotation=ResourceServerSubjectTypeAuthorization,
                    direction="write",
                ),
                "authorization_policy": convert_and_respect_annotation_metadata(
                    object_=authorization_policy,
                    annotation=typing.Optional[ResourceServerAuthorizationPolicy],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateResourceServerResponseContent,
                    parse_obj_as(
                        type_=UpdateResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawResourceServersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        identifiers: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]:
        """
        Retrieve details of all APIs associated with your tenant.

        Parameters
        ----------
        identifiers : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            An optional filter on the resource server identifier. Must be URL encoded and may be specified multiple times (max 10).<br /><b>e.g.</b> <i>../resource-servers?identifiers=id1&identifiers=id2</i>

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ResourceServer, ListResourceServerOffsetPaginatedResponseContent]
            Resource servers successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "resource-servers",
            method="GET",
            params={
                "identifiers": identifiers,
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListResourceServerOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListResourceServerOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.resource_servers
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        identifiers=identifiers,
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        include_fields=include_fields,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        identifier: str,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateResourceServerResponseContent]:
        """
        Create a new API associated with your tenant. Note that all new APIs must be registered with Auth0. For more information, read <a href="https://www.auth0.com/docs/get-started/apis"> APIs</a>.

        Parameters
        ----------
        identifier : str
            Unique identifier for the API used as the audience parameter on authorization calls. Can not be changed once set.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        enforce_policies : typing.Optional[bool]
            Whether to enforce authorization policies (true) or to ignore them (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateResourceServerResponseContent]
            Resource server successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "resource-servers",
            method="POST",
            json={
                "name": name,
                "identifier": identifier,
                "scopes": convert_and_respect_annotation_metadata(
                    object_=scopes, annotation=typing.Sequence[ResourceServerScope], direction="write"
                ),
                "signing_alg": signing_alg,
                "signing_secret": signing_secret,
                "allow_offline_access": allow_offline_access,
                "allow_online_access": allow_online_access,
                "allow_online_access_with_ephemeral_sessions": allow_online_access_with_ephemeral_sessions,
                "token_lifetime": token_lifetime,
                "token_dialect": token_dialect,
                "skip_consent_for_verifiable_first_party_clients": skip_consent_for_verifiable_first_party_clients,
                "enforce_policies": enforce_policies,
                "token_encryption": convert_and_respect_annotation_metadata(
                    object_=token_encryption,
                    annotation=typing.Optional[ResourceServerTokenEncryption],
                    direction="write",
                ),
                "consent_policy": consent_policy,
                "authorization_details": authorization_details,
                "proof_of_possession": convert_and_respect_annotation_metadata(
                    object_=proof_of_possession,
                    annotation=typing.Optional[ResourceServerProofOfPossession],
                    direction="write",
                ),
                "subject_type_authorization": convert_and_respect_annotation_metadata(
                    object_=subject_type_authorization,
                    annotation=ResourceServerSubjectTypeAuthorization,
                    direction="write",
                ),
                "authorization_policy": convert_and_respect_annotation_metadata(
                    object_=authorization_policy,
                    annotation=typing.Optional[ResourceServerAuthorizationPolicy],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateResourceServerResponseContent,
                    parse_obj_as(
                        type_=CreateResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        id: str,
        *,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetResourceServerResponseContent]:
        """
        Retrieve <a href="https://auth0.com/docs/apis">API</a> details with the given ID.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to retrieve.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetResourceServerResponseContent]
            Resource server successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="GET",
            params={
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetResourceServerResponseContent,
                    parse_obj_as(
                        type_=GetResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete an existing API by ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or the audience of the resource server to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        scopes: typing.Optional[typing.Sequence[ResourceServerScope]] = OMIT,
        signing_alg: typing.Optional[SigningAlgorithmEnum] = OMIT,
        signing_secret: typing.Optional[str] = OMIT,
        skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = OMIT,
        allow_offline_access: typing.Optional[bool] = OMIT,
        allow_online_access: typing.Optional[bool] = OMIT,
        allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = OMIT,
        token_lifetime: typing.Optional[int] = OMIT,
        token_dialect: typing.Optional[ResourceServerTokenDialectSchemaEnum] = OMIT,
        enforce_policies: typing.Optional[bool] = OMIT,
        token_encryption: typing.Optional[ResourceServerTokenEncryption] = OMIT,
        consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = OMIT,
        authorization_details: typing.Optional[typing.Sequence[typing.Any]] = OMIT,
        proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = OMIT,
        subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = OMIT,
        authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateResourceServerResponseContent]:
        """
        Change an existing API setting by resource server ID. For more information, read <a href="https://www.auth0.com/docs/get-started/apis/api-settings">API Settings</a>.

        Parameters
        ----------
        id : str
            ID or audience of the resource server to update.

        name : typing.Optional[str]
            Friendly name for this resource server. Can not contain `<` or `>` characters.

        scopes : typing.Optional[typing.Sequence[ResourceServerScope]]
            List of permissions (scopes) that this API uses.

        signing_alg : typing.Optional[SigningAlgorithmEnum]

        signing_secret : typing.Optional[str]
            Secret used to sign tokens when using symmetric algorithms (HS256).

        skip_consent_for_verifiable_first_party_clients : typing.Optional[bool]
            Whether to skip user consent for applications flagged as first party (true) or not (false).

        allow_offline_access : typing.Optional[bool]
            Whether refresh tokens can be issued for this API (true) or not (false).

        allow_online_access : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued for this API (true) or not (false).

        allow_online_access_with_ephemeral_sessions : typing.Optional[bool]
            Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).

        token_lifetime : typing.Optional[int]
            Expiration value (in seconds) for access tokens issued for this API from the token endpoint.

        token_dialect : typing.Optional[ResourceServerTokenDialectSchemaEnum]

        enforce_policies : typing.Optional[bool]
            Whether authorization policies are enforced (true) or not enforced (false).

        token_encryption : typing.Optional[ResourceServerTokenEncryption]

        consent_policy : typing.Optional[ResourceServerConsentPolicyEnum]

        authorization_details : typing.Optional[typing.Sequence[typing.Any]]

        proof_of_possession : typing.Optional[ResourceServerProofOfPossession]

        subject_type_authorization : typing.Optional[ResourceServerSubjectTypeAuthorization]

        authorization_policy : typing.Optional[ResourceServerAuthorizationPolicy]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateResourceServerResponseContent]
            Resource server successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"resource-servers/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "scopes": convert_and_respect_annotation_metadata(
                    object_=scopes, annotation=typing.Sequence[ResourceServerScope], direction="write"
                ),
                "signing_alg": signing_alg,
                "signing_secret": signing_secret,
                "skip_consent_for_verifiable_first_party_clients": skip_consent_for_verifiable_first_party_clients,
                "allow_offline_access": allow_offline_access,
                "allow_online_access": allow_online_access,
                "allow_online_access_with_ephemeral_sessions": allow_online_access_with_ephemeral_sessions,
                "token_lifetime": token_lifetime,
                "token_dialect": token_dialect,
                "enforce_policies": enforce_policies,
                "token_encryption": convert_and_respect_annotation_metadata(
                    object_=token_encryption,
                    annotation=typing.Optional[ResourceServerTokenEncryption],
                    direction="write",
                ),
                "consent_policy": consent_policy,
                "authorization_details": authorization_details,
                "proof_of_possession": convert_and_respect_annotation_metadata(
                    object_=proof_of_possession,
                    annotation=typing.Optional[ResourceServerProofOfPossession],
                    direction="write",
                ),
                "subject_type_authorization": convert_and_respect_annotation_metadata(
                    object_=subject_type_authorization,
                    annotation=ResourceServerSubjectTypeAuthorization,
                    direction="write",
                ),
                "authorization_policy": convert_and_respect_annotation_metadata(
                    object_=authorization_policy,
                    annotation=typing.Optional[ResourceServerAuthorizationPolicy],
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateResourceServerResponseContent,
                    parse_obj_as(
                        type_=UpdateResourceServerResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

ConnectionAgentIpad = str
"""
IP address of the AD connector agent used to validate that authentication requests originate from the corporate network for Kerberos authentication  (managed by the AD Connector agent).
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .user_id import UserId
from .user_profile_data import UserProfileData


class UserIdentity(UniversalBaseModel):
    connection: str = pydantic.Field()
    """
    Connection name of this identity.
    """

    user_id: UserId = pydantic.Field()
    """
    user_id of this identity.
    """

    provider: str = pydantic.Field()
    """
    Type of identity provider.
    """

    profile_data: typing_extensions.Annotated[
        typing.Optional[UserProfileData], FieldMetadata(alias="profileData"), pydantic.Field(alias="profileData")
    ] = None
    is_social: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isSocial"),
        pydantic.Field(
            alias="isSocial", description="Whether the identity provider is a social provider (true) or not (false)."
        ),
    ] = None
    """
    Whether the identity provider is a social provider (true) or not (false).
    """

    access_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    IDP access token returned if scope `read:user_idp_tokens` is defined.
    """

    access_token_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    IDP access token secret returned only if `scope read:user_idp_tokens` is defined.
    """

    refresh_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    IDP refresh token returned only if scope `read:user_idp_tokens` is defined.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_context_client import EventStreamCloudEventContextClient
from .event_stream_cloud_event_context_connection import EventStreamCloudEventContextConnection
from .event_stream_cloud_event_context_request import EventStreamCloudEventContextRequest
from .event_stream_cloud_event_context_tenant import EventStreamCloudEventContextTenant


class EventStreamCloudEventContext(UniversalBaseModel):
    """
    Information about the context in which the event was produced. This may include things like
    HTTP request details, client information, connection information, etc.

    Note: This field may not be present on all events, depending on the event type and the
    context in which it was generated.
    """

    client: typing.Optional[EventStreamCloudEventContextClient] = None
    connection: typing.Optional[EventStreamCloudEventContextConnection] = None
    request: typing.Optional[EventStreamCloudEventContextRequest] = None
    tenant: EventStreamCloudEventContextTenant

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_block_divider_config import FormBlockDividerConfig
from .form_block_type_divider_const import FormBlockTypeDividerConst
from .form_component_category_block_const import FormComponentCategoryBlockConst


class FormBlockDivider(UniversalBaseModel):
    id: str
    category: FormComponentCategoryBlockConst
    type: FormBlockTypeDividerConst
    config: typing.Optional[FormBlockDividerConfig] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_selfServiceProfiles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "self_service_profiles.list_.0"
    client = get_client(test_id)
    client.self_service_profiles.list(
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/self-service-profiles", {"page": "1", "per_page": "1", "include_totals": "true"}, 1
    )


def test_selfServiceProfiles_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "self_service_profiles.create.0"
    client = get_client(test_id)
    client.self_service_profiles.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/self-service-profiles", None, 1)


def test_selfServiceProfiles_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "self_service_profiles.get.0"
    client = get_client(test_id)
    client.self_service_profiles.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/self-service-profiles/id", None, 1)


def test_selfServiceProfiles_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "self_service_profiles.delete.0"
    client = get_client(test_id)
    client.self_service_profiles.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/self-service-profiles/id", None, 1)


def test_selfServiceProfiles_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "self_service_profiles.update.0"
    client = get_client(test_id)
    client.self_service_profiles.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/self-service-profiles/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .client_addon_aws import ClientAddonAws
from .client_addon_azure_blob import ClientAddonAzureBlob
from .client_addon_azure_sb import ClientAddonAzureSb
from .client_addon_box import ClientAddonBox
from .client_addon_cloud_bees import ClientAddonCloudBees
from .client_addon_concur import ClientAddonConcur
from .client_addon_dropbox import ClientAddonDropbox
from .client_addon_echo_sign import ClientAddonEchoSign
from .client_addon_egnyte import ClientAddonEgnyte
from .client_addon_firebase import ClientAddonFirebase
from .client_addon_layer import ClientAddonLayer
from .client_addon_mscrm import ClientAddonMscrm
from .client_addon_new_relic import ClientAddonNewRelic
from .client_addon_oag import ClientAddonOag
from .client_addon_office_365 import ClientAddonOffice365
from .client_addon_rms import ClientAddonRms
from .client_addon_salesforce import ClientAddonSalesforce
from .client_addon_salesforce_api import ClientAddonSalesforceApi
from .client_addon_salesforce_sandbox_api import ClientAddonSalesforceSandboxApi
from .client_addon_saml import ClientAddonSaml
from .client_addon_sapapi import ClientAddonSapapi
from .client_addon_sentry import ClientAddonSentry
from .client_addon_share_point import ClientAddonSharePoint
from .client_addon_slack import ClientAddonSlack
from .client_addon_spring_cm import ClientAddonSpringCm
from .client_addon_sso_integration import ClientAddonSsoIntegration
from .client_addon_wams import ClientAddonWams
from .client_addon_ws_fed import ClientAddonWsFed
from .client_addon_zendesk import ClientAddonZendesk
from .client_addon_zoom import ClientAddonZoom


class ClientAddons(UniversalBaseModel):
    """
    Addons enabled for this client and their associated configurations.
    """

    aws: typing.Optional[ClientAddonAws] = None
    azure_blob: typing.Optional[ClientAddonAzureBlob] = None
    azure_sb: typing.Optional[ClientAddonAzureSb] = None
    rms: typing.Optional[ClientAddonRms] = None
    mscrm: typing.Optional[ClientAddonMscrm] = None
    slack: typing.Optional[ClientAddonSlack] = None
    sentry: typing.Optional[ClientAddonSentry] = None
    box: typing.Optional[ClientAddonBox] = None
    cloudbees: typing.Optional[ClientAddonCloudBees] = None
    concur: typing.Optional[ClientAddonConcur] = None
    dropbox: typing.Optional[ClientAddonDropbox] = None
    echosign: typing.Optional[ClientAddonEchoSign] = None
    egnyte: typing.Optional[ClientAddonEgnyte] = None
    firebase: typing.Optional[ClientAddonFirebase] = None
    newrelic: typing.Optional[ClientAddonNewRelic] = None
    office_365: typing_extensions.Annotated[
        typing.Optional[ClientAddonOffice365], FieldMetadata(alias="office365"), pydantic.Field(alias="office365")
    ] = None
    salesforce: typing.Optional[ClientAddonSalesforce] = None
    salesforce_api: typing.Optional[ClientAddonSalesforceApi] = None
    salesforce_sandbox_api: typing.Optional[ClientAddonSalesforceSandboxApi] = None
    samlp: typing.Optional[ClientAddonSaml] = None
    layer: typing.Optional[ClientAddonLayer] = None
    sap_api: typing.Optional[ClientAddonSapapi] = None
    sharepoint: typing.Optional[ClientAddonSharePoint] = None
    springcm: typing.Optional[ClientAddonSpringCm] = None
    wams: typing.Optional[ClientAddonWams] = None
    wsfed: typing.Optional[ClientAddonWsFed] = None
    zendesk: typing.Optional[ClientAddonZendesk] = None
    zoom: typing.Optional[ClientAddonZoom] = None
    sso_integration: typing.Optional[ClientAddonSsoIntegration] = None
    oag: typing.Optional[ClientAddonOag] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_disable_signup_sms import ConnectionDisableSignupSms
from .connection_forward_req_info_sms import ConnectionForwardReqInfoSms
from .connection_from_sms import ConnectionFromSms
from .connection_gateway_authentication_sms import ConnectionGatewayAuthenticationSms
from .connection_gateway_url_sms import ConnectionGatewayUrlSms
from .connection_messaging_service_sid_sms import ConnectionMessagingServiceSidSms
from .connection_options_common import ConnectionOptionsCommon
from .connection_provider_sms import ConnectionProviderSms
from .connection_template_sms import ConnectionTemplateSms
from .connection_template_syntax_enum_sms import ConnectionTemplateSyntaxEnumSms
from .connection_totp_sms import ConnectionTotpSms
from .connection_twilio_sid_sms import ConnectionTwilioSidSms
from .connection_twilio_token_sms import ConnectionTwilioTokenSms


class ConnectionOptionsSms(ConnectionOptionsCommon):
    """
    Options for the 'sms' connection
    """

    brute_force_protection: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether brute force protection is enabled
    """

    disable_signup: typing.Optional[ConnectionDisableSignupSms] = None
    forward_req_info: typing.Optional[ConnectionForwardReqInfoSms] = None
    from_: typing_extensions.Annotated[
        typing.Optional[ConnectionFromSms], FieldMetadata(alias="from"), pydantic.Field(alias="from")
    ] = None
    gateway_authentication: typing.Optional[ConnectionGatewayAuthenticationSms] = None
    gateway_url: typing.Optional[ConnectionGatewayUrlSms] = None
    messaging_service_sid: typing.Optional[ConnectionMessagingServiceSidSms] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Connection name
    """

    provider: typing.Optional[ConnectionProviderSms] = None
    syntax: typing.Optional[ConnectionTemplateSyntaxEnumSms] = None
    template: typing.Optional[ConnectionTemplateSms] = None
    totp: typing.Optional[ConnectionTotpSms] = None
    twilio_sid: typing.Optional[ConnectionTwilioSidSms] = None
    twilio_token: typing.Optional[ConnectionTwilioTokenSms] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....errors.forbidden_error import ForbiddenError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.get_guardian_factors_provider_apns_response_content import GetGuardianFactorsProviderApnsResponseContent
from ....types.get_guardian_factors_provider_push_notification_response_content import (
    GetGuardianFactorsProviderPushNotificationResponseContent,
)
from ....types.get_guardian_factors_provider_sns_response_content import GetGuardianFactorsProviderSnsResponseContent
from ....types.guardian_factors_provider_push_notification_provider_data_enum import (
    GuardianFactorsProviderPushNotificationProviderDataEnum,
)
from ....types.set_guardian_factors_provider_push_notification_apns_response_content import (
    SetGuardianFactorsProviderPushNotificationApnsResponseContent,
)
from ....types.set_guardian_factors_provider_push_notification_fcm_response_content import (
    SetGuardianFactorsProviderPushNotificationFcmResponseContent,
)
from ....types.set_guardian_factors_provider_push_notification_fcmv_1_response_content import (
    SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
)
from ....types.set_guardian_factors_provider_push_notification_response_content import (
    SetGuardianFactorsProviderPushNotificationResponseContent,
)
from ....types.set_guardian_factors_provider_push_notification_sns_response_content import (
    SetGuardianFactorsProviderPushNotificationSnsResponseContent,
)
from ....types.update_guardian_factors_provider_push_notification_apns_response_content import (
    UpdateGuardianFactorsProviderPushNotificationApnsResponseContent,
)
from ....types.update_guardian_factors_provider_push_notification_fcm_response_content import (
    UpdateGuardianFactorsProviderPushNotificationFcmResponseContent,
)
from ....types.update_guardian_factors_provider_push_notification_fcmv_1_response_content import (
    UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
)
from ....types.update_guardian_factors_provider_push_notification_sns_response_content import (
    UpdateGuardianFactorsProviderPushNotificationSnsResponseContent,
)
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPushNotificationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_apns_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGuardianFactorsProviderApnsResponseContent]:
        """
        Retrieve configuration details for the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGuardianFactorsProviderApnsResponseContent]
            APNS configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderApnsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_apns_provider(
        self,
        *,
        sandbox: typing.Optional[bool] = OMIT,
        bundle_id: typing.Optional[str] = OMIT,
        p_12: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetGuardianFactorsProviderPushNotificationApnsResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        sandbox : typing.Optional[bool]

        bundle_id : typing.Optional[str]

        p_12 : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorsProviderPushNotificationApnsResponseContent]
            APNS configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="PUT",
            json={
                "sandbox": sandbox,
                "bundle_id": bundle_id,
                "p12": p_12,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationApnsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_apns_provider(
        self,
        *,
        sandbox: typing.Optional[bool] = OMIT,
        bundle_id: typing.Optional[str] = OMIT,
        p_12: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateGuardianFactorsProviderPushNotificationApnsResponseContent]:
        """
        Modify configuration details of the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        sandbox : typing.Optional[bool]

        bundle_id : typing.Optional[str]

        p_12 : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateGuardianFactorsProviderPushNotificationApnsResponseContent]
            APNS configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="PATCH",
            json={
                "sandbox": sandbox,
                "bundle_id": bundle_id,
                "p12": p_12,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationApnsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_fcm_provider(
        self, *, server_key: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SetGuardianFactorsProviderPushNotificationFcmResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication FCM provider associated with your tenant.

        Parameters
        ----------
        server_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorsProviderPushNotificationFcmResponseContent]
            FCM configuration updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcm",
            method="PUT",
            json={
                "server_key": server_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationFcmResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationFcmResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_fcm_provider(
        self, *, server_key: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmResponseContent]:
        """
        Modify configuration details of the multi-factor authentication FCM provider associated with your tenant.

        Parameters
        ----------
        server_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmResponseContent]
            FCM configuration updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcm",
            method="PATCH",
            json={
                "server_key": server_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationFcmResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationFcmResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_fcmv_1_provider(
        self,
        *,
        server_credentials: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication FCMV1 provider associated with your tenant.

        Parameters
        ----------
        server_credentials : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent]
            FCMV1 configuration updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcmv1",
            method="PUT",
            json={
                "server_credentials": server_credentials,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_fcmv_1_provider(
        self,
        *,
        server_credentials: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent]:
        """
        Modify configuration details of the multi-factor authentication FCMV1 provider associated with your tenant.

        Parameters
        ----------
        server_credentials : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent]
            FCMV1 configuration updated
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcmv1",
            method="PATCH",
            json={
                "server_credentials": server_credentials,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_sns_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGuardianFactorsProviderSnsResponseContent]:
        """
        Retrieve configuration details for an AWS SNS push notification provider that has been enabled for MFA. To learn more, review [Configure Push Notifications for MFA](https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGuardianFactorsProviderSnsResponseContent]
            AWS SNS configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderSnsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_sns_provider(
        self,
        *,
        aws_access_key_id: typing.Optional[str] = OMIT,
        aws_secret_access_key: typing.Optional[str] = OMIT,
        aws_region: typing.Optional[str] = OMIT,
        sns_apns_platform_application_arn: typing.Optional[str] = OMIT,
        sns_gcm_platform_application_arn: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetGuardianFactorsProviderPushNotificationSnsResponseContent]:
        """
        Configure the [AWS SNS push notification provider configuration](https://auth0.com/docs/multifactor-authentication/developer/sns-configuration) (subscription required).

        Parameters
        ----------
        aws_access_key_id : typing.Optional[str]

        aws_secret_access_key : typing.Optional[str]

        aws_region : typing.Optional[str]

        sns_apns_platform_application_arn : typing.Optional[str]

        sns_gcm_platform_application_arn : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorsProviderPushNotificationSnsResponseContent]
            AWS SNS configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="PUT",
            json={
                "aws_access_key_id": aws_access_key_id,
                "aws_secret_access_key": aws_secret_access_key,
                "aws_region": aws_region,
                "sns_apns_platform_application_arn": sns_apns_platform_application_arn,
                "sns_gcm_platform_application_arn": sns_gcm_platform_application_arn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationSnsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_sns_provider(
        self,
        *,
        aws_access_key_id: typing.Optional[str] = OMIT,
        aws_secret_access_key: typing.Optional[str] = OMIT,
        aws_region: typing.Optional[str] = OMIT,
        sns_apns_platform_application_arn: typing.Optional[str] = OMIT,
        sns_gcm_platform_application_arn: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateGuardianFactorsProviderPushNotificationSnsResponseContent]:
        """
        Configure the [AWS SNS push notification provider configuration](https://auth0.com/docs/multifactor-authentication/developer/sns-configuration) (subscription required).

        Parameters
        ----------
        aws_access_key_id : typing.Optional[str]

        aws_secret_access_key : typing.Optional[str]

        aws_region : typing.Optional[str]

        sns_apns_platform_application_arn : typing.Optional[str]

        sns_gcm_platform_application_arn : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateGuardianFactorsProviderPushNotificationSnsResponseContent]
            AWS SNS configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="PATCH",
            json={
                "aws_access_key_id": aws_access_key_id,
                "aws_secret_access_key": aws_secret_access_key,
                "aws_region": aws_region,
                "sns_apns_platform_application_arn": sns_apns_platform_application_arn,
                "sns_gcm_platform_application_arn": sns_gcm_platform_application_arn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationSnsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_selected_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetGuardianFactorsProviderPushNotificationResponseContent]:
        """
        Modify the push notification provider configured for your tenant. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa">Configure Push Notifications for MFA</a>.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetGuardianFactorsProviderPushNotificationResponseContent]
            Returns selected push notification provider
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/selected-provider",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderPushNotificationResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderPushNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_provider(
        self,
        *,
        provider: GuardianFactorsProviderPushNotificationProviderDataEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SetGuardianFactorsProviderPushNotificationResponseContent]:
        """
        Modify the push notification provider configured for your tenant. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa">Configure Push Notifications for MFA</a>.

        Parameters
        ----------
        provider : GuardianFactorsProviderPushNotificationProviderDataEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SetGuardianFactorsProviderPushNotificationResponseContent]
            Returns selected push notification provider configuration
        """
        _response = self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/selected-provider",
            method="PUT",
            json={
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPushNotificationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_apns_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGuardianFactorsProviderApnsResponseContent]:
        """
        Retrieve configuration details for the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGuardianFactorsProviderApnsResponseContent]
            APNS configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderApnsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_apns_provider(
        self,
        *,
        sandbox: typing.Optional[bool] = OMIT,
        bundle_id: typing.Optional[str] = OMIT,
        p_12: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationApnsResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        sandbox : typing.Optional[bool]

        bundle_id : typing.Optional[str]

        p_12 : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationApnsResponseContent]
            APNS configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="PUT",
            json={
                "sandbox": sandbox,
                "bundle_id": bundle_id,
                "p12": p_12,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationApnsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_apns_provider(
        self,
        *,
        sandbox: typing.Optional[bool] = OMIT,
        bundle_id: typing.Optional[str] = OMIT,
        p_12: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationApnsResponseContent]:
        """
        Modify configuration details of the multi-factor authentication APNS provider associated with your tenant.

        Parameters
        ----------
        sandbox : typing.Optional[bool]

        bundle_id : typing.Optional[str]

        p_12 : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationApnsResponseContent]
            APNS configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/apns",
            method="PATCH",
            json={
                "sandbox": sandbox,
                "bundle_id": bundle_id,
                "p12": p_12,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationApnsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationApnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_fcm_provider(
        self, *, server_key: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationFcmResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication FCM provider associated with your tenant.

        Parameters
        ----------
        server_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationFcmResponseContent]
            FCM configuration updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcm",
            method="PUT",
            json={
                "server_key": server_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationFcmResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationFcmResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_fcm_provider(
        self, *, server_key: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmResponseContent]:
        """
        Modify configuration details of the multi-factor authentication FCM provider associated with your tenant.

        Parameters
        ----------
        server_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmResponseContent]
            FCM configuration updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcm",
            method="PATCH",
            json={
                "server_key": server_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationFcmResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationFcmResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_fcmv_1_provider(
        self,
        *,
        server_credentials: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent]:
        """
        Overwrite all configuration details of the multi-factor authentication FCMV1 provider associated with your tenant.

        Parameters
        ----------
        server_credentials : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent]
            FCMV1 configuration updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcmv1",
            method="PUT",
            json={
                "server_credentials": server_credentials,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationFcmv1ResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_fcmv_1_provider(
        self,
        *,
        server_credentials: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent]:
        """
        Modify configuration details of the multi-factor authentication FCMV1 provider associated with your tenant.

        Parameters
        ----------
        server_credentials : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent]
            FCMV1 configuration updated
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/fcmv1",
            method="PATCH",
            json={
                "server_credentials": server_credentials,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationFcmv1ResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_sns_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGuardianFactorsProviderSnsResponseContent]:
        """
        Retrieve configuration details for an AWS SNS push notification provider that has been enabled for MFA. To learn more, review [Configure Push Notifications for MFA](https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa).

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGuardianFactorsProviderSnsResponseContent]
            AWS SNS configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderSnsResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_sns_provider(
        self,
        *,
        aws_access_key_id: typing.Optional[str] = OMIT,
        aws_secret_access_key: typing.Optional[str] = OMIT,
        aws_region: typing.Optional[str] = OMIT,
        sns_apns_platform_application_arn: typing.Optional[str] = OMIT,
        sns_gcm_platform_application_arn: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationSnsResponseContent]:
        """
        Configure the [AWS SNS push notification provider configuration](https://auth0.com/docs/multifactor-authentication/developer/sns-configuration) (subscription required).

        Parameters
        ----------
        aws_access_key_id : typing.Optional[str]

        aws_secret_access_key : typing.Optional[str]

        aws_region : typing.Optional[str]

        sns_apns_platform_application_arn : typing.Optional[str]

        sns_gcm_platform_application_arn : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationSnsResponseContent]
            AWS SNS configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="PUT",
            json={
                "aws_access_key_id": aws_access_key_id,
                "aws_secret_access_key": aws_secret_access_key,
                "aws_region": aws_region,
                "sns_apns_platform_application_arn": sns_apns_platform_application_arn,
                "sns_gcm_platform_application_arn": sns_gcm_platform_application_arn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationSnsResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_sns_provider(
        self,
        *,
        aws_access_key_id: typing.Optional[str] = OMIT,
        aws_secret_access_key: typing.Optional[str] = OMIT,
        aws_region: typing.Optional[str] = OMIT,
        sns_apns_platform_application_arn: typing.Optional[str] = OMIT,
        sns_gcm_platform_application_arn: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationSnsResponseContent]:
        """
        Configure the [AWS SNS push notification provider configuration](https://auth0.com/docs/multifactor-authentication/developer/sns-configuration) (subscription required).

        Parameters
        ----------
        aws_access_key_id : typing.Optional[str]

        aws_secret_access_key : typing.Optional[str]

        aws_region : typing.Optional[str]

        sns_apns_platform_application_arn : typing.Optional[str]

        sns_gcm_platform_application_arn : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateGuardianFactorsProviderPushNotificationSnsResponseContent]
            AWS SNS configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/providers/sns",
            method="PATCH",
            json={
                "aws_access_key_id": aws_access_key_id,
                "aws_secret_access_key": aws_secret_access_key,
                "aws_region": aws_region,
                "sns_apns_platform_application_arn": sns_apns_platform_application_arn,
                "sns_gcm_platform_application_arn": sns_gcm_platform_application_arn,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateGuardianFactorsProviderPushNotificationSnsResponseContent,
                    parse_obj_as(
                        type_=UpdateGuardianFactorsProviderPushNotificationSnsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_selected_provider(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetGuardianFactorsProviderPushNotificationResponseContent]:
        """
        Modify the push notification provider configured for your tenant. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa">Configure Push Notifications for MFA</a>.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetGuardianFactorsProviderPushNotificationResponseContent]
            Returns selected push notification provider
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/selected-provider",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetGuardianFactorsProviderPushNotificationResponseContent,
                    parse_obj_as(
                        type_=GetGuardianFactorsProviderPushNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_provider(
        self,
        *,
        provider: GuardianFactorsProviderPushNotificationProviderDataEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationResponseContent]:
        """
        Modify the push notification provider configured for your tenant. For more information, review <a href="https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors/configure-push-notifications-for-mfa">Configure Push Notifications for MFA</a>.

        Parameters
        ----------
        provider : GuardianFactorsProviderPushNotificationProviderDataEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SetGuardianFactorsProviderPushNotificationResponseContent]
            Returns selected push notification provider configuration
        """
        _response = await self._client_wrapper.httpx_client.request(
            "guardian/factors/push-notification/selected-provider",
            method="PUT",
            json={
                "provider": provider,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SetGuardianFactorsProviderPushNotificationResponseContent,
                    parse_obj_as(
                        type_=SetGuardianFactorsProviderPushNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_module_dependency import ActionModuleDependency
from .action_module_secret import ActionModuleSecret
from .action_module_version_reference import ActionModuleVersionReference


class CreateActionModuleResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID of the module.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the module.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The source code from the module's draft version.
    """

    dependencies: typing.Optional[typing.List[ActionModuleDependency]] = pydantic.Field(default=None)
    """
    The npm dependencies from the module's draft version.
    """

    secrets: typing.Optional[typing.List[ActionModuleSecret]] = pydantic.Field(default=None)
    """
    The secrets from the module's draft version (names and timestamps only, values never returned).
    """

    actions_using_module_total: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of deployed actions using this module.
    """

    all_changes_published: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether all draft changes have been published as a version.
    """

    latest_version_number: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version number of the latest published version. Omitted if no versions have been published.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp when the module was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp when the module was last updated.
    """

    latest_version: typing.Optional[ActionModuleVersionReference] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_client_id_bitbucket import ConnectionClientIdBitbucket
from .connection_client_secret_bitbucket import ConnectionClientSecretBitbucket
from .connection_options_common import ConnectionOptionsCommon
from .connection_profile_bitbucket import ConnectionProfileBitbucket
from .connection_scope_array import ConnectionScopeArray
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum


class ConnectionOptionsBitbucket(ConnectionOptionsCommon):
    """
    Options for the 'bitbucket' connection
    """

    client_id: typing.Optional[ConnectionClientIdBitbucket] = None
    client_secret: typing.Optional[ConnectionClientSecretBitbucket] = None
    freeform_scopes: typing.Optional[ConnectionScopeArray] = None
    profile: typing.Optional[ConnectionProfileBitbucket] = None
    scope: typing.Optional[ConnectionScopeArray] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
